package de.fhw.komponentenarchitekturen.knauf.jsf.tests;

import java.io.File;
import java.io.IOException;

import org.jboss.arquillian.api.Deployment;
import org.jboss.arquillian.junit.Arquillian;
import org.jboss.jsfunit.api.InitialPage;
import org.jboss.jsfunit.jsfsession.JSFClientSession;
import org.jboss.jsfunit.jsfsession.JSFServerSession;
import org.jboss.jsfunit.jsfsession.JSFSession;
import org.jboss.shrinkwrap.api.ShrinkWrap;
import org.jboss.shrinkwrap.api.asset.EmptyAsset;
import org.jboss.shrinkwrap.api.spec.WebArchive;
import org.jboss.shrinkwrap.resolver.api.DependencyResolvers;
import org.jboss.shrinkwrap.resolver.api.maven.MavenDependencyResolver;
import org.junit.Assert;
import org.junit.Test;
import org.junit.runner.RunWith;

import de.fhw.komponentenarchitekturen.knauf.jsf.GeometricModelHandler;
import de.fhw.komponentenarchitekturen.knauf.jsf.History;
import de.fhw.komponentenarchitekturen.knauf.jsf.SideLengths;

/**Test the "geometricmodel.faces" JSF page and the corresponding managed bean.
 * This test class has two test methods, which both perform the same tests,
 * but one method relies on the @see InitialPage annotation, the other creates
 * the session itself.
 * 
 * @author Wolfgang Knauf
 * 
 */
@RunWith(Arquillian.class)
public class GeometricModelTest
{

  /**Create a deployable war file. The test will run against the app in this WAR file
   * 
   * @return Webarchive with the server side code
   */
  @Deployment
  public static WebArchive createDeployment()
  {
    // Use the same name as "Displayname" in web.xml!!!
    WebArchive war = ShrinkWrap.create(WebArchive.class, "JSF.war");

    war.setWebXML(new File("src/main/webapp/WEB-INF/web.xml"));

    // war.addPackage("de.fhw.komponentenarchitekturen.knauf.jsf"); //Test class will be added, too...
    war.addClasses(GeometricModelHandler.class, History.class, SideLengths.class, GeometricModelTest.class);

    war.addAsWebResource(new File("src/main/webapp/index.jsp"));
    war.addAsWebResource(new File("src/main/webapp/geometricmodel.jsp"));

    war.addAsWebInfResource(new File("src/main/webapp/WEB-INF//faces-config.xml"), "faces-config.xml");

    // Required for CDI Injection...
    war.addAsWebInfResource(EmptyAsset.INSTANCE, "beans.xml");

    //Find "jstl.jar" in your local maven repository!
    File[] jars = DependencyResolvers.use(MavenDependencyResolver.class).artifact("javax.servlet:jstl:1.2").resolveAsFiles();
    war.addAsLibraries (jars);
    
    System.out.println("JSF.GeometricModelTest.createDeployment");

    System.out.println(war.toString(true));

    return war;
  }

  /**
   * Tests the page "geometricmode.jsp": submits the form and checks the calculated values.
   * 
   * @param server
   *          Server side, created by JSFUnit
   * @param client
   *          Client side, created by JSFUnit
   * @throws IOException
   *           If e.g. a client-submit fails.
   */
  @Test
  @InitialPage("/geometricmodel.faces")
  public void testGeometricModel(JSFServerSession server, JSFClientSession client) throws IOException
  {
    // Test navigation to initial viewID

    System.out.println("Entering testGeometricModel"); // + server);
    
    Assert.assertEquals("/geometricmodel.jsp", server.getCurrentViewID());

    // Set data to form:
    client.setValue("formGeometricModelInput:a", "1");
    client.setValue("formGeometricModelInput:b", "2");
    client.setValue("formGeometricModelInput:c", "3");

    // Submit form:
    client.click("formGeometricModelInput:calculate");

    Assert.assertEquals("/geometricmodel.jsp", server.getCurrentViewID());

    // Get results and assert they have the expected values:
    Double volume = (Double) server.getComponentValue("formGeometricModelInput:volume");
    Double surface = (Double) server.getComponentValue("formGeometricModelInput:surface");
    System.out.println("Volume: " + volume + ", Surface: " + surface);

    Assert.assertEquals(6.0, volume.doubleValue(), 0.0);
    Assert.assertEquals(22.0, surface.doubleValue(), 0.0);
  }

  /**
   * Tests the page "geometricmode.jsp": submits the form and checks the calculated values.
   * 
   * Here the session is created by the test method.
   * 
   * @throws IOException
   *           If e.g. a client-submit fails.
   */
  @Test
  public void testGeometricModelAndCreateSession() throws IOException
  {
    // Test navigation to initial viewID

    System.out.println("Entering testGeometricModelAndCreateSession");

    JSFSession jsfSession = new JSFSession("/geometricmodel.faces");

    JSFClientSession client = jsfSession.getJSFClientSession();
    JSFServerSession server = jsfSession.getJSFServerSession();

    Assert.assertEquals("/geometricmodel.jsp", server.getCurrentViewID());

    // Set data to form:
    client.setValue("formGeometricModelInput:a", "1");
    client.setValue("formGeometricModelInput:b", "2");
    client.setValue("formGeometricModelInput:c", "3");

    // Submit form:
    client.click("formGeometricModelInput:calculate");

    Assert.assertEquals("/geometricmodel.jsp", server.getCurrentViewID());

    // Get results and assert they have the expected values:
    Double volume = (Double) server.getComponentValue("formGeometricModelInput:volume");
    Double surface = (Double) server.getComponentValue("formGeometricModelInput:surface");
    System.out.println("Volume: " + volume + ", Surface: " + surface);

    Assert.assertEquals(6.0, volume.doubleValue(), 0.0);
    Assert.assertEquals(22.0, surface.doubleValue(), 0.0);
  }

}
